# -*- coding: utf-8 -*-

import os
import shutil
import tarfile
import urllib.parse

from . import repo_local
from . import repo_online
from . import repo_publish
from .config import LAYERS_DIR, PUB_LAYERS_DIR, REPO_URL

DEFINITION_MEMBERS = {'layers', 'env', 'uid', 'gid', 'cmd', 'cwd', 'ready', 'halt', 'size', 'dlsize', 'hash'}

class Image:
    def __init__(self, name, load_from_repo=True):
        self.name = name
        self.layer_path = os.path.join(LAYERS_DIR, name)
        self.archive_path = os.path.join(PUB_LAYERS_DIR, f'{name}.tar.xz')
        self.online_path = urllib.parse.urljoin(REPO_URL, 'images', f'{name}.tar.xz')
        self.layers = [name]
        self.env = {}
        self.uid = None
        self.gid = None
        self.cmd = None
        self.cwd = None
        self.ready = None
        self.halt = None
        self.size = None
        self.dlsize = None
        self.hash = None
        if load_from_repo:
            self.set_definition(repo_local.get_image(name))

    def set_definition(self, definition):
        for key in DEFINITION_MEMBERS.intersection(definition):
            setattr(self, key, definition[key])

    def get_definition(self):
        definition = {}
        for key in DEFINITION_MEMBERS:
            value = getattr(self, key)
            if value:
                definition[key] = value
        return definition

    def create(self, imagebuilder):
        # Build the container from image file and save to local repository
        # Chown is possible only when the process is running as root, for user namespaces, see https://linuxcontainers.org/lxc/manpages/man1/lxc-usernsexec.1.html
        os.makedirs(self.layer_path, 0o755, True)
        os.chown(self.layer_path, 100000, 100000)
        imagebuilder.process_file()
        repo_local.register_image(self.name, self.get_definition())

    def delete(self):
        repo_local.unregister_image(self.name)
        try:
            shutil.rmtree(self.layer_path)
        except FileNotFoundError:
            pass

    def publish(self):
        os.makedirs(PUB_LAYERS_DIR, 0o755, True)
        files = repo_publish.TarSizeCounter()
        with tarfile.open(self.archive_path, 'w:xz') as tar:
            tar.add(self.layer_path, self.name, filter=files.add_file)
        self.size = files.size
        self.dlsize = os.path.getsize(self.archive_path)
        self.hash = repo_publish.sign_file(self.archive_path).hex()
        repo_publish.register_image(self.name, self.get_definition())

    def unpublish(self):
        repo_publish.unregister_image(self.name)
        try:
            os.unlink(self.archive_path)
        except FileNotFoundError:
            pass

    def download(self):
        definition = repo_online.get_image(self.name)
        repo_online.download_archive(self.online_path, self.layer_path, definition['hash'])
        self.set_definition(definition)
        repo_local.register_image(self.name, definition)
