# -*- coding: utf-8 -*-

import fcntl
import ipaddress
import socket
import struct

from .config import HOSTS_FILE, NETWORK_INTERFACE

# ioctl magic constants taken from https://git.musl-libc.org/cgit/musl/tree/include/sys/ioctl.h (same as glibc)
IOCTL_SIOCGIFADDR = 0x8915
IOCTL_SIOCGIFNETMASK = 0x891b

def read_leases():
    # Read and parse all IP-hostname pairs from the global hosts file
    try:
        with open(HOSTS_FILE, 'r') as f:
            leases = [lease.strip().split(' ', 1) for lease in f]
            return {ip: hostname for ip, hostname in leases}
    except:
        interface = get_bridge_interface()
        return {str(interface.ip): 'host'}

def write_leases(leases):
    # write all IP-hostname pairs to the global hosts file
    with open(HOSTS_FILE, 'w') as f:
        for ip, hostname in sorted(leases.items(), key=lambda lease: socket.inet_aton(lease[0])):
            f.write(f'{ip} {hostname}\n')

def get_bridge_interface():
    # Returns bridge interface's IP address and netmask
    with socket.socket(socket.AF_INET) as sock:
        # Get IPv4Interface for given interface name
        packed_ifname = struct.pack('256s', NETWORK_INTERFACE.encode())
        ip = socket.inet_ntoa(fcntl.ioctl(sock.fileno(), IOCTL_SIOCGIFADDR, packed_ifname)[20:24])
        netmask = socket.inet_ntoa(fcntl.ioctl(sock.fileno(), IOCTL_SIOCGIFNETMASK, packed_ifname)[20:24])
    return ipaddress.IPv4Interface(f'{ip}/{netmask}')

def request_ip(container_name):
    # Find if and IP hasn't been leased for the hostname
    interface = get_bridge_interface()
    leases = read_leases()
    for ip in leases:
        if leases[ip] == container_name:
            return (ip, str(interface.network.prefixlen), str(interface.ip))
    # If not, get the first unassigned IP from the interface's network
    for ip in interface.network.hosts():
        ip = str(ip)
        if ip not in leases:
            leases[ip] = container_name
            write_leases(leases)
            return (ip, str(interface.network.prefixlen), str(interface.ip))

def release_ip(container_name):
    # Delete the lease from hosts file
    leases = {ip: h for ip, h in read_leases().items() if h != container_name}
    write_leases(leases)
