# -*- coding: utf-8 -*-

import hashlib
import json
import requests
import tarfile
import tempfile

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec, utils
from cryptography.hazmat.primitives.serialization import load_pem_public_key

from .exceptions import AppNotFoundError, ImageNotFoundError
from .config import REPO_AUTH, REPO_PUBKEY, REPO_PACKAGES_URL, REPO_SIG_URL

TYPE_APP = 'apps'
TYPE_IMAGE = 'images'

def get_pubkey():
    pubkey = f'-----BEGIN PUBLIC KEY-----\n{REPO_PUBKEY}\n-----END PUBLIC KEY-----'
    return load_pem_public_key(pubkey.encode(), default_backend())

PUBLIC_KEY = get_pubkey()

def download_archive(src, dst, expected_hash):
    # Download archive via http(s), verify hash and decompress
    with tempfile.TemporaryFile() as tmp_archive:
        sha512 = hashes.SHA512()
        hasher = hashes.Hash(sha512, default_backend())
        # Download the file via http(s) and store as temporary file
        with requests.Session(auth=REPO_AUTH) as session:
            resource = session.get(src, stream=True)
            for chunk in resource.iter_content(chunk_size=None):
                if chunk:
                    tmp_archive.write(chunk)
                    hasher.update(chunk)
        # Verify hash
        PUBLIC_KEY.verify(bytes.fromhex(expected_hash), hasher.finalize(), ec.ECDSA(utils.Prehashed(sha512)))
        # Extract the tar.xz file
        tmp_archive.seek(0)
        with tarfile.open(fileobj=tmp_archive) as tar:
            tar.extractall(dst, numeric_owner=True)

def load():
    with requests.Session(auth=REPO_AUTH) as session:
        packages = session.get(REPO_PACKAGES_URL, timout=5).content
        packages_sig = bytes.fromhex(session.get(REPO_SIG_URL, timout=5).content)
    PUBLIC_KEY.verify(packages_sig, packages, ec.ECDSA(hashes.SHA512()))
    return json.loads(packages)

def get_entries(entry_type):
    data = load()
    return data[entry_type]

def get_entry(entry_type, name):
    return get_entries(entry_type)[name]

def get_images():
    return get_entries(TYPE_IMAGE)

def get_image(image_name):
    try:
        return get_entry(TYPE_IMAGE, image_name)
    except KeyError as e:
        raise ImageNotFoundError(image_name) from e

def get_apps():
    return get_entries(TYPE_APP)

def get_app(app_name):
    try:
        return get_entry(TYPE_APP, image_name)
    except KeyError as e:
        raise ImageNotFoundError(image_name) from e
